@@ -44,7 +44,9 @@ struct _GAttrib {
 	GSList *events;
 	guint next_cmd_id;
 	guint next_evt_id;
+	GDestroyNotify destroy;
 	GAttribDisconnectFunc disconnect;
+	gpointer destroy_user_data;
 	gpointer disc_user_data;
 };
 
@@ -173,6 +175,7 @@ void g_attrib_unref(GAttrib *attrib)
 
 	while ((c = g_queue_pop_head(attrib->queue)))
 		command_destroy(c);
+
 	attrib->queue = NULL;
 
 	for (l = attrib->events; l; l = l->next)
@@ -184,10 +187,15 @@ void g_attrib_unref(GAttrib *attrib)
 	if (attrib->write_watch > 0)
 		g_source_remove(attrib->write_watch);
 
-	if (attrib->read_watch > 0)
+	if (attrib->read_watch > 0) {
 		g_source_remove(attrib->read_watch);
+		g_io_channel_unref(attrib->io);
+	}
+
+
+	if (attrib->destroy)
+		attrib->destroy(attrib->destroy_user_data);
 
-	g_io_channel_unref(attrib->io);
 	g_free(attrib);
 }
 
@@ -203,78 +211,14 @@ gboolean g_attrib_set_disconnect_function(GAttrib *attrib,
 	return TRUE;
 }
 
-static void destroy_receiver(gpointer data)
+gboolean g_attrib_set_destroy_function(GAttrib *attrib,
+		GDestroyNotify destroy, gpointer user_data)
 {
-	struct _GAttrib *attrib = data;
-
-	attrib->read_watch = 0;
-
-	if (attrib->disconnect)
-		attrib->disconnect(attrib->disc_user_data);
-}
-
-static void wake_up_sender(struct _GAttrib *attrib);
-
-static gboolean received_data(GIOChannel *io, GIOCondition cond, gpointer data)
-{
-	struct _GAttrib *attrib = data;
-	struct command *cmd = NULL;
-	GSList *l;
-	uint8_t buf[512], status;
-	gsize len;
-	GIOStatus iostat;
-
-	if (cond & (G_IO_HUP | G_IO_ERR | G_IO_NVAL))
+	if (attrib == NULL)
 		return FALSE;
 
-	memset(buf, 0, sizeof(buf));
-
-	iostat = g_io_channel_read_chars(io, (gchar *) buf, sizeof(buf),
-								&len, NULL);
-	if (iostat != G_IO_STATUS_NORMAL) {
-		status = ATT_ECODE_IO;
-		goto done;
-	}
-
-	for (l = attrib->events; l; l = l->next) {
-		struct event *evt = l->data;
-
-		if (evt->expected == buf[0] ||
-					evt->expected == GATTRIB_ALL_EVENTS)
-			evt->func(buf, len, evt->user_data);
-	}
-
-	if (is_response(buf[0]) == FALSE)
-		return TRUE;
-
-	cmd = g_queue_pop_head(attrib->queue);
-	if (cmd == NULL) {
-		/* Keep the watch if we have events to report */
-		return attrib->events != NULL;
-	}
-
-	if (buf[0] == ATT_OP_ERROR) {
-		status = buf[4];
-		goto done;
-	}
-
-	if (cmd->expected != buf[0]) {
-		status = ATT_ECODE_IO;
-		goto done;
-	}
-
-	status = 0;
-
-done:
-	if (attrib->queue && g_queue_is_empty(attrib->queue) == FALSE)
-		wake_up_sender(attrib);
-
-	if (cmd) {
-		if (cmd->func)
-			cmd->func(status, buf, len, cmd->user_data);
-
-		command_destroy(cmd);
-	}
+	attrib->destroy = destroy;
+	attrib->destroy_user_data = user_data;
 
 	return TRUE;
 }
@@ -333,6 +277,74 @@ static void wake_up_sender(struct _GAttrib *attrib)
 			attrib, destroy_sender);
 }
 
+static gboolean received_data(GIOChannel *io, GIOCondition cond, gpointer data)
+{
+	struct _GAttrib *attrib = data;
+	struct command *cmd = NULL;
+	GSList *l;
+	uint8_t buf[512], status;
+	gsize len;
+	GIOStatus iostat;
+
+	if (cond & (G_IO_HUP | G_IO_ERR | G_IO_NVAL)) {
+		attrib->read_watch = 0;
+		if (attrib->disconnect)
+			attrib->disconnect(attrib->disc_user_data);
+		return FALSE;
+	}
+
+	memset(buf, 0, sizeof(buf));
+
+	iostat = g_io_channel_read_chars(io, (gchar *) buf, sizeof(buf),
+								&len, NULL);
+	if (iostat != G_IO_STATUS_NORMAL) {
+		status = ATT_ECODE_IO;
+		goto done;
+	}
+
+	for (l = attrib->events; l; l = l->next) {
+		struct event *evt = l->data;
+
+		if (evt->expected == buf[0] ||
+					evt->expected == GATTRIB_ALL_EVENTS)
+			evt->func(buf, len, evt->user_data);
+	}
+
+	if (is_response(buf[0]) == FALSE)
+		return TRUE;
+
+	cmd = g_queue_pop_head(attrib->queue);
+	if (cmd == NULL) {
+		/* Keep the watch if we have events to report */
+		return attrib->events != NULL;
+	}
+
+	if (buf[0] == ATT_OP_ERROR) {
+		status = buf[4];
+		goto done;
+	}
+
+	if (cmd->expected != buf[0]) {
+		status = ATT_ECODE_IO;
+		goto done;
+	}
+
+	status = 0;
+
+done:
+	if (attrib->queue && g_queue_is_empty(attrib->queue) == FALSE)
+		wake_up_sender(attrib);
+
+	if (cmd) {
+		if (cmd->func)
+			cmd->func(status, buf, len, cmd->user_data);
+
+		command_destroy(cmd);
+	}
+
+	return TRUE;
+}
+
 GAttrib *g_attrib_new(GIOChannel *io)
 {
 	struct _GAttrib *attrib;
@@ -347,10 +359,9 @@ GAttrib *g_attrib_new(GIOChannel *io)
 	attrib->mtu = 512;
 	attrib->queue = g_queue_new();
 
-	attrib->read_watch = g_io_add_watch_full(attrib->io,
-			G_PRIORITY_DEFAULT,
+	attrib->read_watch = g_io_add_watch(attrib->io,
 			G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
-			received_data, attrib, destroy_receiver);
+			received_data, attrib);
 
 	return g_attrib_ref(attrib);
 }
@@ -471,12 +482,54 @@ guint g_attrib_register(GAttrib *attrib, guint8 opcode,
 	return event->id;
 }
 
+static gint event_cmp_by_id(gconstpointer a, gconstpointer b)
+{
+	const struct event *evt = a;
+	guint id = GPOINTER_TO_UINT(b);
+
+	return evt->id - id;
+}
+
 gboolean g_attrib_unregister(GAttrib *attrib, guint id)
 {
+	struct event *evt;
+	GSList *l;
+
+	l = g_slist_find_custom(attrib->events, GUINT_TO_POINTER(id),
+							event_cmp_by_id);
+	if (l == NULL)
+		return FALSE;
+
+	evt = l->data;
+
+	attrib->events = g_slist_remove(attrib->events, evt);
+
+	if (evt->notify)
+		evt->notify(evt->user_data);
+
+	g_free(evt);
+
 	return TRUE;
 }
 
 gboolean g_attrib_unregister_all(GAttrib *attrib)
 {
+	GSList *l;
+
+	if (attrib->events == NULL)
+		return FALSE;
+
+	for (l = attrib->events; l; l = l->next) {
+		struct event *evt = l->data;
+
+		if (evt->notify)
+			evt->notify(evt->user_data);
+
+		g_free(evt);
+	}
+
+	g_slist_free(attrib->events);
+	attrib->events = NULL;
+
 	return TRUE;
 }
